%Arbitrary Sample Rate Conversion (ASRC) by Farrow Interpolator Method
%
%[y t N t0 x0] = asrc_farrow(x, dt, t0, x0, P)
%
%Where:
%   x  = vector of input samples at times [1:length(x)]
%   y  = vector of output samples at times t
%   t  = vector of zero delay output sample times
%   N  = number of samples returned in y and t
%   dt = time step between output samples
%   t0 = time of first output sample in interval [0,dt) on input
%      = time of next first output sample
%   x0 = previous K samples of on input, i.e. [x(-K+1) ... x(0)]
%      = next x0 on output
%   P  = matrix of farrow polynomials
%
%Initial Setup:
%   t0 = 0 (or any value in [0,dt)
%   x0 = zeros(1,K)
%
%Matt Boytim - 9/12/08
%maboytim@yahoo.com

function [y,t,N,t0,x0] = asrc_farrow(x,dt,t0,x0,P)

% get filter and polynolial orders from the size of P
% K = polynomial approximation order
% L = order of Farrow arm filters (L+1 tap)
K = size(P,2)-1;
L = size(P,1)-1;

% the time interval of [x0 x] is assumed to be (1-K):Nx
Nx = length(x);

% number of output points in the in the interval [0:Nx)
N = ceil((Nx-t0)/dt);

% and their sample times
t = t0+[0:N-1]*dt;

% compute input sample times from output sample times
% floor(.)+1 is essentially ceil(.) but with a different boundary case
T = floor(t)+1;

% x(T) is the newest sample involved in the interpolation
% the fractional sample is relatve to the input sample spacing
% T-K      T-4      T-3      T-2      T-1       T
%  +--------+--------+---x----+--------+----o---+
%                                           t
%                                      |<-->|
%                                  f = t-(T-1)
f = t - (T-1);

% do farrow iterations
y = zeros(1,N);
for k=1:K+1,
  y0 = filter(P(:,k),1,[x0 x]);
  y = f.*y + y0(T+L);
end;

% offset t by filter delay to correspond with input
t = t - (L-1)/2;

% compute t0,x0 for next time
t0 = t0 + N*dt - Nx;
x0 = x(Nx-L+1:Nx);
