%Arbitrary Sample Rate Conversion (ASRC) by Farrow Interpolator Method
%
%This performs the same function as asrc_farrow_loop() except with a loop
%instead of Matlab vector operations.
%
%[y t N t0 x0] = asrc_farrow_loop(x, dt, t0, x0, P)
%
%Where:
%   x  = vector of input samples at times [1:length(x)]
%   y  = vector of output samples at times t
%   t  = vector of zero delay output sample times
%   N  = number of samples returned in y and t
%   dt = time step between output samples
%   t0 = time of first output sample in interval [0,dt) on input
%      = time of next first output sample
%   x0 = previous K samples of on input, i.e. [x(-K+1) ... x(0)]
%      = next x0 on output
%   P  = matrix of farrow polynomials
%
%Initial Setup:
%   t0 = 0 (or any value in [0,dt)
%   x0 = zeros(1,K)
%
%Matt Boytim - 9/21/08
%maboytim@yahoo.com

function [y,t,N,t0,x0] = asrc_farrow_loop(x,dt,t0,x0,P)

% get filter and polynolial orders from the size of P
% K = polynomial approximation order
% L = order of Farrow arm filters (L+1 tap)
K = size(P,2)-1;
L = size(P,1)-1;

% the time interval of [x0 x] is assumed to be (1-K):Nx
Nx = length(x);

% augment input with initial state
x = [x0 x];

% number of output points
N = 0;

% for all possible outputs
while t0<Nx;
  % get the integer sample index
  T = floor(t0)+1;

  % and the fraction
  f = t0 - (T-1);

  % perform farrow iteration
  y0 = 0;
  for k=1:K+1,
    y0 = f*y0 + x(T:T+L)*P(L+1:-1:1,k);
  end;

  % save output time and output
  t = [t t0];
  y = [y y0];

  % bump output count
  N = N+1;

  % bump output time
  t0 = t0 + dt;
end;

% offset t by filter delay to correspond with input
t = t - (L-1)/2;

% initial conditions for next time
t0 = t0 - Nx;
x0 = x(Nx+1:Nx+L);
