%Construct the Farrow polynomials from a prototype filter.  The prototype filter
%is 'oversampled' so a polynomial can be fit through the given points.  The output
%matrix P of polynomial coefficients can then be used with the asrc_farrow() function
%to perform arbitrary sample rate conversion using Farrow's method.
%
%Usage:
%[P E] = construct_farrow(h,L,K)
%
%Where:
%   h  = the impulse response of the oversampled protype filter
%   L  = the primative order of h
%        the length of h should be L*M+1 for some integer M where M is the
%          oversampling ratio of h - that is, you can think of h as an L order
%          filter (L+1 taps) with M-1 values in between each one
%   K  = approximation polynomial order
%   P  = LxK matrix of polynomials that approximate h
%   E  = approximation error vector such that h+E is the approximated response
%
%Matt Boytim - 9/12/08
%maboytim@yahoo.com

function [P,E] = construct_farrow(h,L,K)

M = (length(h)-1)/L;

% create the piecewise polynomial approximation to the desired filter
% the polynomials for each segment are gathered in P
% the approximation error is gathered in E
P = [];
E = [];
for l=1:L,
  x = [0:M]/M;
  y = h((l-1)*M+1:l*M+1);
  p = polyfit(x,y,K);
  P = [P; p];
  % evaluate the approximation polynomial over the interval and compute the error
  e = polyval(p,x)-y;
  E = [E e(1:M)];
end;
E = [E e(M+1)];