%Farrow interpolator impulse response.
%
%Conceptually the interploation can be viewed as reconstruction of the input sequence
%to continuous time using the interploation filter for the reconstruction then
%resampling the continuous signal at the output rate.  The passband response of the
%interpolation filter will color the signal so ideally should have constant amplitude
%and delay in the passband.  The stopband response of the interpolation filter should
%suppress the images of the input signal so as not to alias in the subsequent
%resampling.  In other words, the interploation should approximate the ideal
%reconstruction lowpass as expected.
%
%Here, the impulse response of the filter is calculated by upsampling an impulse to
%some arbitrarily high multiple M of the input sample rate as an approximation to
%real time.

% number of frequency domain sample points within input nyquist
% (controls plotting resolution)
np = 100;

% L = primative filter order (L+1 tap filter)
L = 6;

% The prototype filter is piecewise polynomial approximated using a least squares
% polynomial fit through M points.
M = 64;

% K = polynomial order
% loop through a bunch
for K=1:25,
  K

  % design the M oversampled prototype filter
  % the oversampling is so a polynomial can be fit through the points
  % the filter length should be L*M+1 for some integer M, the L*M order guarantees it
  % the filter response itself depends on the application, this is just an example
  h = remez(L*M,[0 .5/M 1.5/M 1],[1 1 0 0]);
  h = h/max(h);

  % create the piecewise polynomial approximation to the desired filter
  [P e] = construct_farrow(h,L,K);

  % the approximated filter is the prototype plus the error
  hh = h+e;
  
  % plot the prototype filter
  subplot(5,1,1);
  plot(h);
  hold on;
  % overlay the polynomial approximation
  plot(hh,'g');
  hold off;
  % plot the prototype frequency response
  [H f] = freqz(h,M,M*np);
  subplot(5,1,2);
  plot(f,max(20*log10(abs(H)),-60));
  hold on;
  % overlay the approximated frequency response
  HH = freqz(hh,M,M*np);
  plot(f,max(20*log10(abs(HH)),-60),'g');
  hold off;

  % create the input impulse
  x = zeros(1,L);
  x(1) = 1;

  % set the initial conditions
  x0 = zeros(1,size(P,1)-1);
  t0 = 0;

  % now upsample the impulse to the same rate as used in the original design
  % the upsampled impulse and the approximated response should be the same
  if 1,
    [h t N t0 x0] = asrc_farrow(x,1/M,t0,x0,P);
  else,
    [h t N t0 x0] = asrc_farrow_loop(x,1/M,t0,x0,P);
  end;

  % plot the approximated impulse response and overlay the filter output
  subplot(5,1,3);
  plot(hh);
  hold on;
  plot(h,'g');
  hold off;

  % plot the magnitude response
  [H,f] = freqz(h/M,1,M*np);
  subplot(5,1,4);
  % first drop a marker at the original nyquist
  plot([pi/M pi/M],[-60 0],'g');
  hold on;
  % plot dB magnitude with a -60dB floor
  plot(f,max(20*log10(abs(H)),-60));
  % zoom in on the passband
  plot(f(1:np)*M,max(20*log10(abs(H(1:np))),-60/10)*10,'r');
  hold off;

  % plot the phase response
  subplot(5,1,5);
  plot(f,angle(H));
  hold on;
  % zoom in on the passband
  plot(f(1:np)*M,angle(H(1:np)),'r');
  hold off;

  pause;
end;